@@ -26,31 +26,35 @@ def warning(*args):
     logger.warning(' '.join(args))
 
 
+def error(*args):
+    logger.error(' '.join(args))
+
+
 def json2geojson(data, filter_used_refs=True, log_level='ERROR'):
     if isinstance(data, str):
         data = json.loads(data)
-    return _json2geojson(data, filter_used_refs)
+    return _json2geojson(data, filter_used_refs, log_level)
 
 
 def xml2geojson(xml_str, filter_used_refs=True, log_level='ERROR'):
     data = parse_xml(xml_str)
-    return _json2geojson(data, filter_used_refs)
+    return _json2geojson(data, filter_used_refs, log_level)
 
 
 def json2shapes(data, filter_used_refs=True, log_level='ERROR'):
     if isinstance(data, str):
         data = json.loads(data)
-    return _json2shapes(data, filter_used_refs)
+    return _json2shapes(data, filter_used_refs, log_level)
 
 
 def xml2shapes(xml_str, filter_used_refs=True, log_level='ERROR'):
     data = parse_xml(xml_str)
-    return _json2shapes(data, filter_used_refs)
+    return _json2shapes(data, filter_used_refs, log_level)
 
 
 def _json2geojson(data, filter_used_refs=True, log_level='ERROR'):
     features = []
-    for shape in _json2shapes(data, filter_used_refs):
+    for shape in _json2shapes(data, filter_used_refs, log_level):
         feature = shape_to_feature(shape['shape'], shape['properties'])
         features.append(feature)
 
@@ -291,10 +295,13 @@ def is_geometry_polygon_without_exceptions(node):
 
 
 def relation_to_shape(rel, refs_index):
-    if is_geometry_polygon(rel):
-        return multipolygon_relation_to_shape(rel, refs_index)
-    else:
-        return multiline_realation_to_shape(rel, refs_index)
+    try:
+        if is_geometry_polygon(rel):
+            return multipolygon_relation_to_shape(rel, refs_index)
+        else:
+            return multiline_realation_to_shape(rel, refs_index)
+    except:
+        error('Failed to convert relation to shape', pformat(rel))
 
 
 def multiline_realation_to_shape(rel, refs_index):
@@ -309,7 +316,8 @@ def multiline_realation_to_shape(rel, refs_index):
         if member['type'] == 'way':
             way_shape = way_to_shape(member, refs_index)
         elif member['type'] == 'relation':
-            refs_index[member['ref']]['used'] = rel['id']
+            if member['ref'] in refs_index:
+                refs_index[member['ref']]['used'] = rel['id']
             way_shape = element_to_shape(member, refs_index)
         else:
             warning('multiline member not handled', pformat(member))
